﻿Imports System.Collections.Generic
Imports System.Drawing

Public Class Canvas : Inherits LayerListMember

    Public Overloads Sub BreakTile(ByVal smashArea As RectangleF)
        Dim tileSize As SizeF = BrokenTile.ApexesToRectangleF(New BrokenTile(PointF.Empty).Apexes).Size

        Dim leftTop As New PointF(smashArea.X - smashArea.X Mod tileSize.Width, smashArea.Y - smashArea.Y Mod tileSize.Height)
        'Dim fitPoint As New PointF(smashArea.X, smashArea.Y)
        Dim fitPoint As New PointF(leftTop.X, leftTop.Y)

        Dim tile As New BrokenTile(PointF.Empty)

        Do While fitPoint.Y <= smashArea.Y + smashArea.Height - 1
            Do While fitPoint.X <= smashArea.X + smashArea.Width - 1
                tile = BreakTile(fitPoint)

                fitPoint.X = fitPoint.X + tileSize.Width
            Loop

            fitPoint.X = leftTop.X
            fitPoint.Y = fitPoint.Y + tileSize.Height
        Loop

        Dim rightBottom As New PointF(smashArea.X + smashArea.Width - 1, smashArea.Y + smashArea.Height - 1)
        Debug.Assert(IIf(Me.Image.GetBounds(GraphicsUnit.Pixel).Contains(rightBottom), BrokenTile.ApexesToRectangleF(tile.Apexes).Contains(rightBottom), True))

        'TODO:さらに上位のCanvasに対しても再帰的にBreakTile呼び出し

    End Sub

    Public Overloads Function BreakTile(ByVal smashLocation As PointF) As BrokenTile
        Dim tileSize As SizeF = BrokenTile.ApexesToRectangleF(New BrokenTile(PointF.Empty).Apexes).Size

        Dim idx As Integer = Int(smashLocation.X / tileSize.Width) + 1
        idx = Int(smashLocation.Y / tileSize.Height + 1) * idx

        'TODO:locationによってidxが決まるように。既存タイルのlocationを指定された場合、生成しないように。上書きするように。
        Dim pt As New PointF(smashLocation.X - smashLocation.X Mod tileSize.Width, smashLocation.Y - smashLocation.Y Mod tileSize.Height)
        Dim tile As New BrokenTile(pt)
        Me._brokenTiles.Add(tile)

        Return tile
    End Function

    Public Sub BreakAllTiles()
        Dim tileSize As SizeF = BrokenTile.ApexesToRectangleF(New BrokenTile(PointF.Empty).Apexes).Size
        Dim fitPoint As New PointF(0, 0)
        Do While fitPoint.X < Me._image.Size.Height
            Do While fitPoint.X < Me._image.Size.Width
                Dim tile As New BrokenTile(fitPoint)
                Me._brokenTiles.Add(tile)
                fitPoint.X = fitPoint.X + tileSize.Width
            Loop
            fitPoint.X = 0
            fitPoint.Y = fitPoint.Y + tileSize.Height
        Loop
    End Sub

    Dim _brokenTiles As List(Of BrokenTile)
    Public Function GetBrokenTiles() As List(Of BrokenTile)
        Return Me._brokenTiles
    End Function

    Public Sub FitTiles(ByVal tiles As List(Of BrokenTile))
        'TODO:将来Tile内のBitmapを無くして、Layer→Canvasへ直接描画することも

        ' Canvasの更新
        Using g As Graphics = Utils.GraphicsFromImage(Me.Image)
            For Each tile In tiles
                g.DrawImage(tile.ImageClip, Rectangle.Round(BrokenTile.ApexesToRectangleF(tile.Apexes)))
            Next
        End Using

    End Sub

    Public Sub ClearBrokenTiles()
        Me._brokenTiles = New List(Of BrokenTile)
        Debug.Assert(Me._brokenTiles.Count = 0)
    End Sub

    Private _image As Bitmap
    Public Overloads ReadOnly Property Image As Bitmap
        Get
            Return Me._image
        End Get
    End Property

    Private _layers As New List(Of LayerListMember)
    Public Property Layers As List(Of LayerListMember)
        Get
            Return Me._layers
        End Get
        Set(ByVal value As List(Of LayerListMember))
            Me._layers = value
        End Set
    End Property

    Public Sub New(ByVal container As Canvas)
        Dim dispRect As Rectangle = Screen.GetWorkingArea(Utils.tGetApp())
        Me._image = New Bitmap(dispRect.Width, dispRect.Height)
        Me._container = container
        Me._brokenTiles = New List(Of BrokenTile)
    End Sub

    Private _container As Canvas
    Public Property Container As Canvas
        Get
            Return Me._container
        End Get
        Set(ByVal value As Canvas)
            Me._container = value
        End Set
    End Property

    Private Shared Sub _serialize(ByVal canvas As Canvas, ByRef l As List(Of LayerListMember))
        For Each member In canvas.Layers
            If member.GetType().Name = canvas.GetType().Name Then
                _serialize(DirectCast(member, Canvas), l)
            Else
                l.Add(member)
            End If
        Next
    End Sub

    Public Function DownwardMember() As List(Of LayerListMember)
        Dim ret As New List(Of LayerListMember)()

        _serialize(Me, ret)

        Return ret

    End Function

    Public Function UpwardMember() As List(Of LayerListMember)
        Dim ret As New List(Of LayerListMember)()

        _serialize(Me, ret)
        ret.Reverse()

        Return ret

    End Function

    Public Overrides Sub Accept(ByVal visitor As Visitor)
        visitor.Visit(Me)
    End Sub

End Class
