﻿Imports System.Drawing
Imports System.Collections.Generic

'Namespace Brushes
Namespace Plugin.Brushes

    Public Class Brush

        Public Overridable Sub Draw(ByVal strokePoints As Queue(Of StrokePoint), ByVal layer As RasterLayer)
            Using g As Graphics = layer.CreateGraphics()
                'g.PageUnit = GraphicsUnit.Pixel

                Do While strokePoints.Count > 0
                    Dim strokePoint As StrokePoint = strokePoints.Dequeue()
                    ' 描画
                    With Nothing
                        Dim width As Single = 100 * strokePoint.Pressure ' width:100px * 筆圧(%)
                        Dim pt As PointF = PointF.Add(New PointF(strokePoint.X, strokePoint.Y), New SizeF(width / -2, width / -2))        'ポインター位置が中心になるよう描くため
                        g.FillPie(Drawing.Brushes.DarkSlateBlue, pt.X, pt.Y, width, width, 0, 360)   ' 座標としてSingle型を渡せるのでこうしているが、特に意味はない。

                        layer.BreakTile(New RectangleF(pt.X, pt.Y, width, width))

                        'TODO:負荷軽減オプション選択時、スプライトレイヤーに描画。表示更新は別途行う。

                    End With
                Loop
            End Using
        End Sub

    End Class

    Public Class BrushCommand : Inherits Command : Implements DrawFreehandLine

        Private _strokePoints As Queue(Of StrokePoint)
        Private _layer As RasterLayer

        Public Overridable Property StrokePoints As System.Collections.Generic.Queue(Of StrokePoint) Implements DrawFreehandLine.StrokePoints
            Get
                Return Me._strokePoints
            End Get
            Set(ByVal value As System.Collections.Generic.Queue(Of StrokePoint))
                Me._strokePoints = value
            End Set
        End Property

        Public Overridable Property Layer As RasterLayer Implements DrawFreehandLine.Layer
            Get
                Return Me._layer
            End Get
            Set(ByVal value As RasterLayer)
                Me._layer = value
            End Set
        End Property

        Public Overridable Sub InStroke(ByVal layer As RasterLayer) Implements DrawFreehandLine.InStroke
            Debug.Assert(Not IsNothing(Me._strokePoints) And Not IsNothing(Me._layer))
            Dim b As New Plugin.Brushes.Brush()

            b.Draw(Me._strokePoints, layer)
        End Sub

        Public Overloads Sub Visit(ByVal canvas As Canvas) Implements Visitor.Visit
            For Each member In canvas.DownwardMember()
                member.Accept(Me)
            Next
        End Sub

        Public Overloads Sub Visit(ByVal layer As FilterLayer) Implements Visitor.Visit
            ' do nothing
        End Sub

        Public Overloads Sub Visit(ByVal layer As RasterLayer) Implements Visitor.Visit
            Debug.Assert(Me.AppEvent = "InStroke" Or Me.AppEvent = "BeginStroke" Or Me.AppEvent = "EndStroke")

            If Me.AppEvent = "InStroke" Then
                Me.InStroke(layer)
            ElseIf Me.AppEvent = "EndStroke" Then
                Me.EndStroke(layer)
            Else
                Debug.Fail("")
            End If
        End Sub

        Private _appEvent As String
        Public Overrides Property AppEvent As String
            Get
                Return Me._appEvent
            End Get
            Set(ByVal value As String)
                Me._appEvent = value
            End Set
        End Property

        Public Sub EndStroke(ByVal layer As RasterLayer) Implements DrawFreehandLine.EndStroke
        End Sub
    End Class

End Namespace
