﻿Imports Microsoft.StylusInput
Imports Microsoft.Ink

Public Class AsyncPlugin : Implements Microsoft.StylusInput.IStylusAsyncPlugin

    Dim px_pt1000 As Point
    Dim inc_pt1000 As Point

    Public ReadOnly Property DataInterest As Microsoft.StylusInput.DataInterestMask Implements Microsoft.StylusInput.IStylusAsyncPlugin.DataInterest
        Get
            '種々のデータのうちどんな情報を使用可能にするか（IStylusSyncPluginで実装可能なメソッドのうち、どれを使用可能にするか）
            Return DataInterestMask.AllStylusData And Not DataInterestMask.SystemGesture
        End Get
    End Property

    Public Sub CustomStylusDataAdded(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.CustomStylusData) Implements Microsoft.StylusInput.IStylusAsyncPlugin.CustomStylusDataAdded
        Debug.Print(Now & " CustomStylusDataAdded")
    End Sub

    Public Sub [Error](ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.ErrorData) Implements Microsoft.StylusInput.IStylusAsyncPlugin.Error
        Debug.Print(Now & " Error: " & data.InnerException.Message)
    End Sub

    Public Sub InAirPackets(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.InAirPacketsData) Implements Microsoft.StylusInput.IStylusAsyncPlugin.InAirPackets
        '入力可能領域でペン先スイッチoff時。ずっと、ペンを動かさなくても繰り返し繰り返し…
        'Debug.Print(Now & " InAirPackets")
    End Sub

    Public Sub Packets(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.PacketsData) Implements Microsoft.StylusInput.IStylusAsyncPlugin.Packets
        '入力可能領域でペン先スイッチon時。ずっと、ペンを動かさなくても繰り返し繰り返し…
        'Debug.Print(Now & " Packets")

        Debug.Assert(data.PacketPropertyCount >= 2)   ' 2～3種類。X, Y, Pressure。おそらくタブレットドライバーのアップデートによって増えていくはず。
        Debug.Assert(data.Count Mod data.PacketPropertyCount = 0)

        Dim strokePoints As New Generic.Queue(Of StrokePoint)

        Using g As Graphics = App.CanvasPanel.CreateGraphics()
            For idx As Integer = 0 To data.Count - (1 * data.PacketPropertyCount) Step data.PacketPropertyCount
                Dim ink_pt As New Point(data(idx), data(idx + 1))

                Dim t As Microsoft.Ink.Tablet = New Microsoft.Ink.Tablets().DefaultTablet

                Dim normalPressure As Microsoft.Ink.TabletPropertyMetrics = Nothing
                Dim pressure As Single                                              ' 筆圧(%)
                If data.PacketPropertyCount >= 3 Then
                    normalPressure = t.GetPropertyMetrics(Microsoft.Ink.PacketProperty.NormalPressure)
                    pressure = data(idx + 2) / normalPressure.Maximum
                Else
                    pressure = 0.1                                                  ' 筆圧が無いときは筆圧固定(%)
                End If

                If pressure > 0 Then
                    ' 入力座標をpx単位（小数含む）に変換
                    Dim per_pt As New PointF(ink_pt.X / inc_pt1000.X * px_pt1000.X, ink_pt.Y / inc_pt1000.Y * px_pt1000.Y)
                    Dim pt As New StrokePoint(per_pt, pressure)
                    strokePoints.Enqueue(pt)
                End If
            Next
        End Using

        App.CanvasPanel.CanvasPanel_Packets(strokePoints)

    End Sub

    Public Sub RealTimeStylusDisabled(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.RealTimeStylusDisabledData) Implements Microsoft.StylusInput.IStylusAsyncPlugin.RealTimeStylusDisabled
        'アプリケーションの終了処理に
        Debug.Print(Now & " RealTimeStylusDisabled")
    End Sub

    Public Sub RealTimeStylusEnabled(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.RealTimeStylusEnabledData) Implements Microsoft.StylusInput.IStylusAsyncPlugin.RealTimeStylusEnabled
        'アプリケーションの初期設定に
        Debug.Print(Now & " RealTimeStylusEnabled")
    End Sub

    Public Sub StylusButtonDown(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.StylusButtonDownData) Implements Microsoft.StylusInput.IStylusAsyncPlugin.StylusButtonDown
        'サイドスイッチ以外に、ペン先スイッチもボタンの1つになっている
        'Debug.Print(Now & " StylusButtonDown")
    End Sub

    Public Sub StylusButtonUp(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.StylusButtonUpData) Implements Microsoft.StylusInput.IStylusAsyncPlugin.StylusButtonUp
        'サイドスイッチ以外に、ペン先スイッチもボタンの1つになっている
        'Debug.Print(Now & " StylusButtonUp")
    End Sub

    Public Sub StylusDown(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.StylusDownData) Implements Microsoft.StylusInput.IStylusAsyncPlugin.StylusDown
        'ペン先スイッチon
        'Debug.Print(Now & " StylusDown")

        App.CanvasPanel.CanvasPanel_StylusDown()

    End Sub

    Public Sub StylusUp(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.StylusUpData) Implements Microsoft.StylusInput.IStylusAsyncPlugin.StylusUp
        'ペン先スイッチoff
        'Debug.Print(Now & " StylusUp")

        App.CanvasPanel.CanvasPanel_StylusUp()

    End Sub

    Public Sub StylusInRange(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.StylusInRangeData) Implements Microsoft.StylusInput.IStylusAsyncPlugin.StylusInRange
        'ペン先スイッチoffで入力可能領域にペンポインターが入ったとき
        'Debug.Print(Now & " StylusInRange")
    End Sub

    Public Sub StylusOutOfRange(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.StylusOutOfRangeData) Implements Microsoft.StylusInput.IStylusAsyncPlugin.StylusOutOfRange
        'ペン先スイッチoffで入力可能領域からペンポインターが出たとき
        'Debug.Print(Now & " StylusOutOfRange")
    End Sub

    Public Sub SystemGesture(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.SystemGestureData) Implements Microsoft.StylusInput.IStylusAsyncPlugin.SystemGesture
        'Debug.Print(Now & " SystemGesture")
    End Sub

    Public Sub TabletAdded(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.TabletAddedData) Implements Microsoft.StylusInput.IStylusAsyncPlugin.TabletAdded
        Debug.Print(Now & " TabletAdded")
    End Sub

    Public Sub TabletRemoved(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.TabletRemovedData) Implements Microsoft.StylusInput.IStylusAsyncPlugin.TabletRemoved
        Debug.Print(Now & " TabletRemoved")
    End Sub

    Public Sub New()
        ' px単位の基準座標設定
        px_pt1000 = New Point(1000, 1000)   ' 座標は型の精度に見合う値なら何でもいい。

        ' px基準座標に対応するインクスペース座標算出
        inc_pt1000 = New Point(px_pt1000)
        Using g As Graphics = App.CanvasPanel.CreateGraphics()
            Dim r As New Renderer()
            r.PixelToInkSpace(g, inc_pt1000)    ' 非常に重い。コスト高な処理。
        End Using

    End Sub

End Class
